@@ -137,6 +137,23 @@ fn main() -> Result<(), corosync::CsError> {
         return Err(e);
     }
 
+    // Check that fd_get returns a valid FD
+    match cpg::fd_get(&handle) {
+        Ok(fd) => {
+            println!("FD is {fd}");
+            // Arbitrary upper limit but FDs should always be low
+            // and we're mainly checking addresses being returned
+            if !(0..=0xFFFF).contains(&fd) {
+                println!("Error - bad fd returned from fd_get: {fd}");
+                return Err(corosync::CsError::CsErrRustCompat);
+            }
+        }
+        Err(e) => {
+            println!("Error in CPG fd_get: {e}");
+            return Err(e);
+        }
+    }
+
     // Quick test of dispatch
     if let Err(e) = cpg::dispatch(&handle, corosync::DispatchFlags::OneNonblocking) {
         println!("Error in CPG dispatch: {e}");
