@@ -35,6 +35,10 @@
 
 #include <config.h>
 
+#ifdef HAVE_ALLOCA_H
+#include <alloca.h>
+#endif
+
 #include <sys/types.h>
 #include <sys/uio.h>
 #include <sys/socket.h>
@@ -628,9 +632,8 @@ static void remove_ro_entries(icmap_map_t temp_map)
 	delete_and_notify_if_changed(temp_map, "totem.version");
 	delete_and_notify_if_changed(temp_map, "totem.threads");
 	delete_and_notify_if_changed(temp_map, "totem.ip_version");
-	delete_and_notify_if_changed(temp_map, "totem.rrp_mode");
+	delete_and_notify_if_changed(temp_map, "totem.ip_dscp");
 	delete_and_notify_if_changed(temp_map, "totem.netmtu");
-	delete_and_notify_if_changed(temp_map, "totem.interface.ringnumber");
 	delete_and_notify_if_changed(temp_map, "totem.interface.bindnetaddr");
 	delete_and_notify_if_changed(temp_map, "totem.interface.mcastaddr");
 	delete_and_notify_if_changed(temp_map, "totem.interface.broadcast");
@@ -722,6 +725,9 @@ static void message_handler_req_exec_cfg_reload_config (
 
 	log_printf(LOGSYS_LEVEL_NOTICE, "Config reload requested by node " CS_PRI_NODE_ID, nodeid);
 
+	// Clear this out in case it all goes well
+	icmap_delete("config.reload_error_message");
+
 	icmap_set_uint8("config.totemconfig_reload_in_progress", 1);
 
 	/* Make sure there is no rubbish in this that might be checked, even on error */
@@ -824,7 +830,6 @@ static void message_handler_req_exec_cfg_reload_config (
 	/* Copy into live system */
 	totempg_put_config(&new_config);
 	totemconfig_commit_new_params(&new_config, temp_map);
-	free(new_config.interfaces);
 
 reload_fini:
 	/* All done - let clients know */
@@ -833,6 +838,7 @@ reload_fini:
 	icmap_set_uint8("config.reload_in_progress", 0);
 
 	/* Finished with the temporary storage */
+	free(new_config.interfaces);
 	free(new_config.orig_interfaces);
 
 reload_fini_nofree:
@@ -1372,15 +1378,14 @@ static void message_handler_req_lib_cfg_get_node_addrs (void *conn,
 {
 	struct totem_ip_address node_ifs[INTERFACE_MAX];
 	unsigned int iface_ids[INTERFACE_MAX];
-	char buf[PIPE_BUF];
 	char **status;
 	unsigned int num_interfaces = 0;
-	struct sockaddr_storage *ss;
 	int ret = CS_OK;
 	int i;
 	int live_addrs = 0;
 	const struct req_lib_cfg_get_node_addrs *req_lib_cfg_get_node_addrs = msg;
-	struct res_lib_cfg_get_node_addrs *res_lib_cfg_get_node_addrs = (struct res_lib_cfg_get_node_addrs *)buf;
+	struct res_lib_cfg_get_node_addrs *res_lib_cfg_get_node_addrs;
+	size_t res_lib_cfg_get_node_addrs_size;
 	unsigned int nodeid = req_lib_cfg_get_node_addrs->nodeid;
 	char *addr_buf;
 
@@ -1392,15 +1397,18 @@ static void message_handler_req_lib_cfg_get_node_addrs (void *conn,
 		num_interfaces = 0;
 	}
 
-	res_lib_cfg_get_node_addrs->header.size = sizeof(struct res_lib_cfg_get_node_addrs) + (num_interfaces * TOTEMIP_ADDRLEN);
+	res_lib_cfg_get_node_addrs_size = sizeof(struct res_lib_cfg_get_node_addrs) + (num_interfaces * TOTEMIP_ADDRLEN);
+	res_lib_cfg_get_node_addrs = alloca(res_lib_cfg_get_node_addrs_size);
+	memset(res_lib_cfg_get_node_addrs, 0, res_lib_cfg_get_node_addrs_size);
+
+	res_lib_cfg_get_node_addrs->header.size = res_lib_cfg_get_node_addrs_size;
 	res_lib_cfg_get_node_addrs->header.id = MESSAGE_RES_CFG_GET_NODE_ADDRS;
 	res_lib_cfg_get_node_addrs->header.error = ret;
 	if (num_interfaces) {
 		res_lib_cfg_get_node_addrs->family = node_ifs[0].family;
 		for (i = 0, addr_buf = (char *)res_lib_cfg_get_node_addrs->addrs;
 		    i < num_interfaces; i++) {
-			ss = (struct sockaddr_storage *)&node_ifs[i].addr;
-			if (ss->ss_family) {
+			if (node_ifs[i].family) {
 				memcpy(addr_buf, node_ifs[i].addr, TOTEMIP_ADDRLEN);
 				live_addrs++;
 				addr_buf += TOTEMIP_ADDRLEN;
