@@ -70,6 +70,7 @@ enum parser_cb_type {
 	PARSER_CB_SECTION_START,
 	PARSER_CB_SECTION_END,
 	PARSER_CB_ITEM,
+	PARSER_CB_CLEANUP,
 };
 
 enum main_cp_cb_data_state {
@@ -271,6 +272,9 @@ static char *remove_whitespace(char *string, int remove_colon_and_brace)
 	char *end;
 
 	start = string;
+	if (*start == '\0')
+		return start;
+
 	while (*start == ' ' || *start == '\t' || (unsigned char)*start == 0xA0)
 		start++;
 
@@ -288,7 +292,7 @@ static char *remove_whitespace(char *string, int remove_colon_and_brace)
 static int parse_section(FILE *fp,
 			const char *fname,
 			int *line_no,
-			char *path,
+			const char *path,
 			const char **error_string,
 			int depth,
 			enum main_cp_cb_data_state state,
@@ -408,6 +412,11 @@ static int parse_section(FILE *fp,
 			key = remove_whitespace(line, 1);
 			value = remove_whitespace(loc, 0);
 
+			if (strlen(key) == 0) {
+				tmp_error_string = "Key name can't be empty";
+				goto parse_error;
+			}
+
 			if (strlen(path) + strlen(key) + 1 >= ICMAP_KEYNAME_MAXLEN) {
 				tmp_error_string = "New key makes total cmap path too long";
 				goto parse_error;
@@ -463,6 +472,7 @@ static int parse_section(FILE *fp,
 
 	if (strcmp(path, "") == 0) {
 		parser_cb("", NULL, NULL, &state, PARSER_CB_END, error_string, config_map, user_data);
+		parser_cb("", NULL, NULL, &state, PARSER_CB_CLEANUP, error_string, config_map, user_data);
 	}
 
 	return 0;
@@ -475,6 +485,8 @@ parse_error:
 		*error_string = formated_err;
 	}
 
+	parser_cb("", NULL, NULL, &state, PARSER_CB_CLEANUP, error_string, config_map, user_data);
+
 	return -1;
 }
 
@@ -556,6 +568,41 @@ static int str_to_ull(const char *str, unsigned long long int *res)
 	return (0);
 }
 
+static int get_dscp_value(char *str, int *dscp) {
+	struct dscp_name {const char *name; int dscp;} names[] = {
+		{"cs0", 0}, {"cs1", 8}, {"cs2", 16}, {"cs3", 24},
+		{"cs4", 32}, {"cs5", 40}, {"cs6", 48}, {"cs7", 56},
+		{"af11", 10}, {"af12", 12}, {"af13", 14}, {"af21", 18},
+		{"af22", 20}, {"af23", 22}, {"af31", 26}, {"af32", 28},
+		{"af33", 30}, {"af41", 34}, {"af42", 36}, {"af43", 38},
+		{"ef", 46}, {NULL, 0}
+	};
+	struct dscp_name *n;
+	long val;
+	char *end;
+
+	/*
+	 * allow dscp symbolical names according to
+	 * https://www.iana.org/assignments/dscp-registry/dscp-registry.xhtml
+	 */
+	for (n = names; n->name; n++) {
+		if (strcmp(str, n->name) == 0) {
+			*dscp = n->dscp;
+			return 0;
+		}
+	}
+
+	/* allow dscp as number (decimal, hex, octal) */
+	errno = 0;
+	val = strtol(str, &end, 0);
+	if (errno == 0 && *end == '\0' && val >= 0 && val <= 63) {
+		*dscp = val;
+		return 0;
+	}
+
+	return -1;
+}
+
 static int handle_crypto_model(const char *val, const char **error_string)
 {
 
@@ -598,8 +645,9 @@ static int main_config_parser_cb(const char *path,
 	struct main_cp_cb_data *data = (struct main_cp_cb_data *)user_data;
 	struct key_value_list_item *kv_item;
 	struct qb_list_head *iter, *tmp_iter;
-	int uid, gid;
+	int uid, gid, dscp;
 	cs_error_t cs_err;
+	const char *path_prefix;
 
 	cs_err = CS_OK;
 
@@ -624,10 +672,39 @@ static int main_config_parser_cb(const char *path,
 	switch (type) {
 	case PARSER_CB_START:
 		memset(data, 0, sizeof(struct main_cp_cb_data));
+		qb_list_init(&data->logger_subsys_items_head);
+		qb_list_init(&data->member_items_head);
 		*state = MAIN_CP_CB_DATA_STATE_NORMAL;
 		break;
 	case PARSER_CB_END:
 		break;
+	case PARSER_CB_CLEANUP:
+		free(data->bindnetaddr);
+		free(data->mcastaddr);
+		free(data->broadcast);
+		free(data->knet_transport);
+
+		qb_list_for_each_safe(iter, tmp_iter, &(data->logger_subsys_items_head)) {
+			kv_item = qb_list_entry(iter, struct key_value_list_item, list);
+			qb_list_del(&kv_item->list);
+
+			free(kv_item->value);
+			free(kv_item->key);
+			free(kv_item);
+		}
+
+		free(data->subsys);
+		free(data->logging_daemon_name);
+
+		qb_list_for_each_safe(iter, tmp_iter, &(data->member_items_head)) {
+			kv_item = qb_list_entry(iter, struct key_value_list_item, list);
+			qb_list_del(&kv_item->list);
+
+			free(kv_item->value);
+			free(kv_item->key);
+			free(kv_item);
+		}
+		break;
 	case PARSER_CB_ITEM:
 		add_as_string = 1;
 
@@ -639,7 +716,7 @@ static int main_config_parser_cb(const char *path,
 			    (strcmp(path, "pload.size") == 0)) {
 				val_type = ICMAP_VALUETYPE_UINT32;
 				if (safe_atoq(value, &val, val_type) != 0) {
-					goto atoi_error;
+					goto safe_atoq_error;
 				}
 				if ((cs_err = icmap_set_uint32_r(config_map, path, val)) != CS_OK) {
 					goto icmap_set_error;
@@ -654,7 +731,7 @@ static int main_config_parser_cb(const char *path,
 			    (strcmp(path, "quorum.leaving_timeout") == 0)) {
 				val_type = ICMAP_VALUETYPE_UINT32;
 				if (safe_atoq(value, &val, val_type) != 0) {
-					goto atoi_error;
+					goto safe_atoq_error;
 				}
 				if ((cs_err = icmap_set_uint32_r(config_map, path, val)) != CS_OK) {
 					goto icmap_set_error;
@@ -670,7 +747,7 @@ static int main_config_parser_cb(const char *path,
 			    (strcmp(path, "quorum.last_man_standing") == 0)) {
 				val_type = ICMAP_VALUETYPE_UINT8;
 				if (safe_atoq(value, &val, val_type) != 0) {
-					goto atoi_error;
+					goto safe_atoq_error;
 				}
 				if ((cs_err = icmap_set_uint8_r(config_map, path, val)) != CS_OK) {
 					goto icmap_set_error;
@@ -684,7 +761,7 @@ static int main_config_parser_cb(const char *path,
 			    (strcmp(path, "quorum.device.votes") == 0)) {
 				val_type = ICMAP_VALUETYPE_UINT32;
 				if (safe_atoq(value, &val, val_type) != 0) {
-					goto atoi_error;
+					goto safe_atoq_error;
 				}
 				if ((cs_err = icmap_set_uint32_r(config_map, path, val)) != CS_OK) {
 					goto icmap_set_error;
@@ -694,7 +771,7 @@ static int main_config_parser_cb(const char *path,
 			if ((strcmp(path, "quorum.device.master_wins") == 0)) {
 				val_type = ICMAP_VALUETYPE_UINT8;
 				if (safe_atoq(value, &val, val_type) != 0) {
-					goto atoi_error;
+					goto safe_atoq_error;
 				}
 				if ((cs_err = icmap_set_uint8_r(config_map, path, val)) != CS_OK) {
 					goto icmap_set_error;
@@ -719,11 +796,6 @@ static int main_config_parser_cb(const char *path,
 			    (strcmp(path, "totem.downcheck") == 0) ||
 			    (strcmp(path, "totem.fail_recv_const") == 0) ||
 			    (strcmp(path, "totem.seqno_unchanged_const") == 0) ||
-			    (strcmp(path, "totem.rrp_token_expired_timeout") == 0) ||
-			    (strcmp(path, "totem.rrp_problem_count_timeout") == 0) ||
-			    (strcmp(path, "totem.rrp_problem_count_threshold") == 0) ||
-			    (strcmp(path, "totem.rrp_problem_count_mcast_threshold") == 0) ||
-			    (strcmp(path, "totem.rrp_autorecovery_check_timeout") == 0) ||
 			    (strcmp(path, "totem.heartbeat_failures_allowed") == 0) ||
 			    (strcmp(path, "totem.max_network_delay") == 0) ||
 			    (strcmp(path, "totem.window_size") == 0) ||
@@ -735,7 +807,7 @@ static int main_config_parser_cb(const char *path,
 			    (strcmp(path, "totem.netmtu") == 0)) {
 				val_type = ICMAP_VALUETYPE_UINT32;
 				if (safe_atoq(value, &val, val_type) != 0) {
-					goto atoi_error;
+					goto safe_atoq_error;
 				}
 				if ((cs_err = icmap_set_uint32_r(config_map,path, val)) != CS_OK) {
 					goto icmap_set_error;
@@ -745,7 +817,7 @@ static int main_config_parser_cb(const char *path,
 			if (strcmp(path, "totem.knet_compression_level") == 0) {
 				val_type = ICMAP_VALUETYPE_INT32;
 				if (safe_atoq(value, &val, val_type) != 0) {
-					goto atoi_error;
+					goto safe_atoq_error;
 				}
 				if ((cs_err = icmap_set_int32_r(config_map, path, val)) != CS_OK) {
 					goto icmap_set_error;
@@ -754,7 +826,7 @@ static int main_config_parser_cb(const char *path,
 			}
 			if (strcmp(path, "totem.config_version") == 0) {
 				if (str_to_ull(value, &ull) != 0) {
-					goto atoi_error;
+					goto str_to_ull_error;
 				}
 				if ((cs_err = icmap_set_uint64_r(config_map, path, ull)) != CS_OK) {
 					goto icmap_set_error;
@@ -808,6 +880,16 @@ static int main_config_parser_cb(const char *path,
 				}
 			}
 
+			if (strcmp(path, "totem.ip_dscp") == 0) {
+				if (get_dscp_value(value, &dscp) != 0) {
+					goto str_to_dscp_error;
+				}
+				if ((cs_err = icmap_set_uint8_r(config_map, path, dscp)) != CS_OK) {
+					goto icmap_set_error;
+				}
+				add_as_string = 0;
+			}
+
 			break;
 
 		case MAIN_CP_CB_DATA_STATE_SYSTEM:
@@ -851,28 +933,31 @@ static int main_config_parser_cb(const char *path,
 			if (strcmp(path, "totem.interface.linknumber") == 0) {
 				val_type = ICMAP_VALUETYPE_UINT8;
 				if (safe_atoq(value, &val, val_type) != 0) {
-					goto atoi_error;
+					goto safe_atoq_error;
 				}
 
 				data->linknumber = val;
 				add_as_string = 0;
 			}
 			if (strcmp(path, "totem.interface.bindnetaddr") == 0) {
+				free(data->bindnetaddr);
 				data->bindnetaddr = strdup(value);
 				add_as_string = 0;
 			}
 			if (strcmp(path, "totem.interface.mcastaddr") == 0) {
+				free(data->mcastaddr);
 				data->mcastaddr = strdup(value);
 				add_as_string = 0;
 			}
 			if (strcmp(path, "totem.interface.broadcast") == 0) {
+				free(data->broadcast);
 				data->broadcast = strdup(value);
 				add_as_string = 0;
 			}
 			if (strcmp(path, "totem.interface.mcastport") == 0) {
 				val_type = ICMAP_VALUETYPE_UINT16;
 				if (safe_atoq(value, &val, val_type) != 0) {
-					goto atoi_error;
+					goto safe_atoq_error;
 				}
 				data->mcastport = val;
 				add_as_string = 0;
@@ -880,7 +965,7 @@ static int main_config_parser_cb(const char *path,
 			if (strcmp(path, "totem.interface.ttl") == 0) {
 				val_type = ICMAP_VALUETYPE_UINT8;
 				if (safe_atoq(value, &val, val_type) != 0) {
-					goto atoi_error;
+					goto safe_atoq_error;
 				}
 				data->ttl = val;
 				add_as_string = 0;
@@ -888,7 +973,7 @@ static int main_config_parser_cb(const char *path,
 			if (strcmp(path, "totem.interface.knet_link_priority") == 0) {
 				val_type = ICMAP_VALUETYPE_UINT8;
 				if (safe_atoq(value, &val, val_type) != 0) {
-					goto atoi_error;
+					goto safe_atoq_error;
 				}
 				data->knet_link_priority = val;
 				add_as_string = 0;
@@ -896,7 +981,7 @@ static int main_config_parser_cb(const char *path,
 			if (strcmp(path, "totem.interface.knet_ping_interval") == 0) {
 				val_type = ICMAP_VALUETYPE_UINT32;
 				if (safe_atoq(value, &val, val_type) != 0) {
-					goto atoi_error;
+					goto safe_atoq_error;
 				}
 				data->knet_ping_interval = val;
 				add_as_string = 0;
@@ -904,7 +989,7 @@ static int main_config_parser_cb(const char *path,
 			if (strcmp(path, "totem.interface.knet_ping_timeout") == 0) {
 				val_type = ICMAP_VALUETYPE_UINT32;
 				if (safe_atoq(value, &val, val_type) != 0) {
-					goto atoi_error;
+					goto safe_atoq_error;
 				}
 				data->knet_ping_timeout = val;
 				add_as_string = 0;
@@ -912,7 +997,7 @@ static int main_config_parser_cb(const char *path,
 			if (strcmp(path, "totem.interface.knet_ping_precision") == 0) {
 				val_type = ICMAP_VALUETYPE_UINT32;
 				if (safe_atoq(value, &val, val_type) != 0) {
-					goto atoi_error;
+					goto safe_atoq_error;
 				}
 				data->knet_ping_precision = val;
 				add_as_string = 0;
@@ -920,19 +1005,20 @@ static int main_config_parser_cb(const char *path,
 			if (strcmp(path, "totem.interface.knet_pong_count") == 0) {
 				val_type = ICMAP_VALUETYPE_UINT32;
 				if (safe_atoq(value, &val, val_type) != 0) {
-					goto atoi_error;
+					goto safe_atoq_error;
 				}
 				data->knet_pong_count = val;
 				add_as_string = 0;
 			}
 			if (strcmp(path, "totem.interface.knet_transport") == 0) {
-				val_type = ICMAP_VALUETYPE_STRING;
+				free(data->knet_transport);
 				data->knet_transport = strdup(value);
 				add_as_string = 0;
 			}
 			break;
 		case MAIN_CP_CB_DATA_STATE_LOGGER_SUBSYS:
-			if (strcmp(key, "subsys") == 0) {
+			if (strcmp(path, "logging.logger_subsys.subsys") == 0) {
+				free(data->subsys);
 				data->subsys = strdup(value);
 				if (data->subsys == NULL) {
 					*error_string = "Can't alloc memory";
@@ -940,6 +1026,14 @@ static int main_config_parser_cb(const char *path,
 					return (0);
 				}
 			} else {
+				path_prefix = "logging.logger_subsys.";
+				if (strlen(path) < strlen(path_prefix) ||
+				    strncmp(path, path_prefix, strlen(path_prefix)) != 0) {
+					*error_string = "Internal error - incorrect path prefix for logger subsys state";
+
+					return (0);
+				}
+
 				kv_item = malloc(sizeof(*kv_item));
 				if (kv_item == NULL) {
 					*error_string = "Can't alloc memory";
@@ -948,7 +1042,7 @@ static int main_config_parser_cb(const char *path,
 				}
 				memset(kv_item, 0, sizeof(*kv_item));
 
-				kv_item->key = strdup(key);
+				kv_item->key = strdup(path + strlen(path_prefix));
 				kv_item->value = strdup(value);
 				if (kv_item->key == NULL || kv_item->value == NULL) {
 					free(kv_item->key);
@@ -964,14 +1058,16 @@ static int main_config_parser_cb(const char *path,
 			add_as_string = 0;
 			break;
 		case MAIN_CP_CB_DATA_STATE_LOGGING_DAEMON:
-			if (strcmp(key, "subsys") == 0) {
+			if (strcmp(path, "logging.logging_daemon.subsys") == 0) {
+				free(data->subsys);
 				data->subsys = strdup(value);
 				if (data->subsys == NULL) {
 					*error_string = "Can't alloc memory";
 
 					return (0);
 				}
-			} else if (strcmp(key, "name") == 0) {
+			} else if (strcmp(path, "logging.logging_daemon.name") == 0) {
+				free(data->logging_daemon_name);
 				data->logging_daemon_name = strdup(value);
 				if (data->logging_daemon_name == NULL) {
 					*error_string = "Can't alloc memory";
@@ -979,6 +1075,14 @@ static int main_config_parser_cb(const char *path,
 					return (0);
 				}
 			} else {
+				path_prefix = "logging.logging_daemon.";
+				if (strlen(path) < strlen(path_prefix) ||
+				    strncmp(path, path_prefix, strlen(path_prefix)) != 0) {
+					*error_string = "Internal error - incorrect path prefix for logging daemon state";
+
+					return (0);
+				}
+
 				kv_item = malloc(sizeof(*kv_item));
 				if (kv_item == NULL) {
 					*error_string = "Can't alloc memory";
@@ -987,7 +1091,7 @@ static int main_config_parser_cb(const char *path,
 				}
 				memset(kv_item, 0, sizeof(*kv_item));
 
-				kv_item->key = strdup(key);
+				kv_item->key = strdup(path + strlen(path_prefix));
 				kv_item->value = strdup(value);
 				if (kv_item->key == NULL || kv_item->value == NULL) {
 					free(kv_item->key);
@@ -1003,7 +1107,7 @@ static int main_config_parser_cb(const char *path,
 			add_as_string = 0;
 			break;
 		case MAIN_CP_CB_DATA_STATE_UIDGID:
-			if (strcmp(key, "uid") == 0) {
+			if (strcmp(path, "uidgid.uid") == 0) {
 				uid = uid_determine(value);
 				if (uid == -1) {
 					*error_string = error_string_response;
@@ -1015,7 +1119,7 @@ static int main_config_parser_cb(const char *path,
 					goto icmap_set_error;
 				}
 				add_as_string = 0;
-			} else if (strcmp(key, "gid") == 0) {
+			} else if (strcmp(path, "uidgid.gid") == 0) {
 				gid = gid_determine(value);
 				if (gid == -1) {
 					*error_string = error_string_response;
@@ -1033,7 +1137,7 @@ static int main_config_parser_cb(const char *path,
 			}
 			break;
 		case MAIN_CP_CB_DATA_STATE_MEMBER:
-			if (strcmp(key, "memberaddr") != 0) {
+			if (strcmp(path, "totem.interface.member.memberaddr") != 0) {
 				*error_string = "Only memberaddr is allowed in member section";
 
 				return (0);
@@ -1064,12 +1168,21 @@ static int main_config_parser_cb(const char *path,
 		case MAIN_CP_CB_DATA_STATE_NODELIST:
 			break;
 		case MAIN_CP_CB_DATA_STATE_NODELIST_NODE:
-			snprintf(key_name, ICMAP_KEYNAME_MAXLEN, "nodelist.node.%u.%s", data->node_number, key);
-			if ((strcmp(key, "nodeid") == 0) ||
-			    (strcmp(key, "quorum_votes") == 0)) {
+			path_prefix = "nodelist.node.";
+			if (strlen(path) < strlen(path_prefix) ||
+			    strncmp(path, path_prefix, strlen(path_prefix)) != 0) {
+				*error_string = "Internal error - incorrect path prefix for nodelist node state";
+
+				return (0);
+			}
+
+			snprintf(key_name, ICMAP_KEYNAME_MAXLEN, "nodelist.node.%u.%s", data->node_number,
+			    path + strlen(path_prefix));
+			if ((strcmp(path, "nodelist.node.nodeid") == 0) ||
+			    (strcmp(path, "nodelist.node.quorum_votes") == 0)) {
 				val_type = ICMAP_VALUETYPE_UINT32;
 				if (safe_atoq(value, &val, val_type) != 0) {
-					goto atoi_error;
+					goto safe_atoq_error;
 				}
 
 				if ((cs_err = icmap_set_uint32_r(config_map, key_name, val)) != CS_OK) {
@@ -1089,7 +1202,7 @@ static int main_config_parser_cb(const char *path,
 			if (strcmp(key, "watchdog_timeout") == 0) {
 				val_type = ICMAP_VALUETYPE_UINT32;
 				if (safe_atoq(value, &val, val_type) != 0) {
-					goto atoi_error;
+					goto safe_atoq_error;
 				}
 				if ((cs_err = icmap_set_uint32_r(config_map,path, val)) != CS_OK) {
 					goto icmap_set_error;
@@ -1101,7 +1214,7 @@ static int main_config_parser_cb(const char *path,
 		case MAIN_CP_CB_DATA_STATE_RESOURCES_SYSTEM_MEMUSED:
 			if (strcmp(key, "poll_period") == 0) {
 				if (str_to_ull(value, &ull) != 0) {
-					goto atoi_error;
+					goto str_to_ull_error;
 				}
 				if ((cs_err = icmap_set_uint64_r(config_map,path, ull)) != CS_OK) {
 					goto icmap_set_error;
@@ -1113,7 +1226,7 @@ static int main_config_parser_cb(const char *path,
 		case MAIN_CP_CB_DATA_STATE_RESOURCES_PROCESS_MEMUSED:
 			if (strcmp(key, "poll_period") == 0) {
 				if (str_to_ull(value, &ull) != 0) {
-					goto atoi_error;
+					goto str_to_ull_error;
 				}
 				if ((cs_err = icmap_set_uint64_r(config_map,path, ull)) != CS_OK) {
 					goto icmap_set_error;
@@ -1194,10 +1307,29 @@ static int main_config_parser_cb(const char *path,
 		if (strcmp(path, "resources.process.memory_used") == 0) {
 			*state = MAIN_CP_CB_DATA_STATE_RESOURCES_PROCESS_MEMUSED;
 		}
+
+		if (*state == MAIN_CP_CB_DATA_STATE_UIDGID && strcmp(path, "uidgid") != 0) {
+			*error_string = "Subsections are not allowed within uidgid section";
+
+			return (0);
+		};
+
+		if (*state == MAIN_CP_CB_DATA_STATE_MEMBER && strcmp(path, "totem.interface.member") != 0) {
+			*error_string = "Subsections are not allowed within totem.interface.member section";
+
+			return (0);
+		};
 		break;
 	case PARSER_CB_SECTION_END:
 		switch (*state) {
 		case MAIN_CP_CB_DATA_STATE_INTERFACE:
+			if (strcmp(path, "totem.interface") != 0) {
+				/*
+				 * Process only end of totem.interface section, not subsections
+				 */
+				break;
+			}
+
 			/*
 			 * Create new interface section
 			 */
@@ -1301,6 +1433,7 @@ static int main_config_parser_cb(const char *path,
 						data->linknumber);
 				cs_err = icmap_set_string_r(config_map, key_name, data->knet_transport);
 				free(data->knet_transport);
+				data->knet_transport = NULL;
 
 				if (cs_err != CS_OK) {
 					goto icmap_set_error;
@@ -1311,6 +1444,7 @@ static int main_config_parser_cb(const char *path,
 
 			qb_list_for_each_safe(iter, tmp_iter, &(data->member_items_head)) {
 				kv_item = qb_list_entry(iter, struct key_value_list_item, list);
+				qb_list_del(&kv_item->list);
 
 				snprintf(key_name, ICMAP_KEYNAME_MAXLEN, "totem.interface.%u.member.%u",
 						data->linknumber, ii);
@@ -1326,8 +1460,17 @@ static int main_config_parser_cb(const char *path,
 				}
 			}
 
+			qb_list_init(&data->member_items_head);
+
 			break;
 		case MAIN_CP_CB_DATA_STATE_LOGGER_SUBSYS:
+			if (strcmp(path, "logging.logger_subsys") != 0) {
+				/*
+				 * Process only end of logging.logger_subsys section, not subsections
+				 */
+				break;
+			}
+
 			if (data->subsys == NULL) {
 				*error_string = "No subsys key in logger_subsys directive";
 
@@ -1336,6 +1479,7 @@ static int main_config_parser_cb(const char *path,
 
 			qb_list_for_each_safe(iter, tmp_iter, &(data->logger_subsys_items_head)) {
 				kv_item = qb_list_entry(iter, struct key_value_list_item, list);
+				qb_list_del(&kv_item->list);
 
 				snprintf(key_name, ICMAP_KEYNAME_MAXLEN, "logging.logger_subsys.%s.%s",
 					 data->subsys, kv_item->key);
@@ -1354,13 +1498,22 @@ static int main_config_parser_cb(const char *path,
 					data->subsys);
 			cs_err = icmap_set_string_r(config_map, key_name, data->subsys);
 
+			qb_list_init(&data->logger_subsys_items_head);
 			free(data->subsys);
+			data->subsys = NULL;
 
 			if (cs_err != CS_OK) {
 				goto icmap_set_error;
 			}
 			break;
 		case MAIN_CP_CB_DATA_STATE_LOGGING_DAEMON:
+			if (strcmp(path, "logging.logging_daemon") != 0) {
+				/*
+				 * Process only end of logging.logging_daemon section, not subsections
+				 */
+				break;
+			}
+
 			if (data->logging_daemon_name == NULL) {
 				*error_string = "No name key in logging_daemon directive";
 
@@ -1369,6 +1522,7 @@ static int main_config_parser_cb(const char *path,
 
 			qb_list_for_each_safe(iter, tmp_iter, &(data->logger_subsys_items_head)) {
 				kv_item = qb_list_entry(iter, struct key_value_list_item, list);
+				qb_list_del(&kv_item->list);
 
 				if (data->subsys == NULL) {
 					if (strcmp(data->logging_daemon_name, "corosync") == 0) {
@@ -1422,9 +1576,6 @@ static int main_config_parser_cb(const char *path,
 					cs_err = icmap_set_string_r(config_map, key_name, data->subsys);
 
 					if (cs_err != CS_OK) {
-						free(data->subsys);
-						free(data->logging_daemon_name);
-
 						goto icmap_set_error;
 					}
 					snprintf(key_name, ICMAP_KEYNAME_MAXLEN, "logging.logging_daemon.%s.%s.name",
@@ -1433,14 +1584,24 @@ static int main_config_parser_cb(const char *path,
 				}
 			}
 
+			qb_list_init(&data->logger_subsys_items_head);
 			free(data->subsys);
+			data->subsys = NULL;
 			free(data->logging_daemon_name);
+			data->logging_daemon_name = NULL;
 
 			if (cs_err != CS_OK) {
 				goto icmap_set_error;
 			}
 			break;
 		case MAIN_CP_CB_DATA_STATE_NODELIST_NODE:
+			if (strcmp(path, "nodelist.node") != 0) {
+				/*
+				 * Process only end of nodelist.node section, not subsections
+				 */
+				break;
+			}
+
 			data->node_number++;
 			break;
 		case MAIN_CP_CB_DATA_STATE_NORMAL:
@@ -1452,21 +1613,11 @@ static int main_config_parser_cb(const char *path,
 		case MAIN_CP_CB_DATA_STATE_NODELIST:
 		case MAIN_CP_CB_DATA_STATE_TOTEM:
 		case MAIN_CP_CB_DATA_STATE_SYSTEM:
-			break;
 		case MAIN_CP_CB_DATA_STATE_RESOURCES:
-			*state = MAIN_CP_CB_DATA_STATE_NORMAL;
-			break;
 		case MAIN_CP_CB_DATA_STATE_RESOURCES_SYSTEM:
-			*state = MAIN_CP_CB_DATA_STATE_RESOURCES;
-			break;
 		case MAIN_CP_CB_DATA_STATE_RESOURCES_SYSTEM_MEMUSED:
-			*state = MAIN_CP_CB_DATA_STATE_RESOURCES_SYSTEM;
-			break;
 		case MAIN_CP_CB_DATA_STATE_RESOURCES_PROCESS:
-			*state = MAIN_CP_CB_DATA_STATE_RESOURCES;
-			break;
 		case MAIN_CP_CB_DATA_STATE_RESOURCES_PROCESS_MEMUSED:
-			*state = MAIN_CP_CB_DATA_STATE_RESOURCES_PROCESS;
 			break;
 		}
 		break;
@@ -1474,7 +1625,10 @@ static int main_config_parser_cb(const char *path,
 
 	return (1);
 
-atoi_error:
+safe_atoq_error:
+	/*
+	 * For integers supported by safe_atoq display range
+	 */
 	min_val = max_val = 0;
 	/*
 	 * This is really assert, because developer ether doesn't set val_type correctly or
@@ -1492,6 +1646,31 @@ atoi_error:
 
 	return (0);
 
+str_to_ull_error:
+	/*
+	 * For integers not supported by safe_atoq (64-bit int)
+	 */
+	if (snprintf(formated_err, sizeof(formated_err),
+	    "Value of key \"%s\" is expected to be unsigned integer, but \"%s\" was given",
+	    key_name, value) >= sizeof(formated_err)) {
+		*error_string = "Can't format parser error message";
+	} else {
+		*error_string = formated_err;
+	}
+
+	return (0);
+
+str_to_dscp_error:
+	if (snprintf(formated_err, sizeof(formated_err),
+	    "Value of key \"%s\" is expected to be number (0..63) or symbolical dscp value, "
+	    "but \"%s\" was given", key_name, value) >= sizeof(formated_err)) {
+		*error_string = "Can't format parser error message";
+	} else {
+		*error_string = formated_err;
+	}
+
+	return (0);
+
 icmap_set_error:
 	if (snprintf(formated_err, sizeof(formated_err),
 	    "Can't store key \"%s\" into icmap, returned error is %s",
@@ -1523,6 +1702,8 @@ static int uidgid_config_parser_cb(const char *path,
 		break;
 	case PARSER_CB_END:
 		break;
+	case PARSER_CB_CLEANUP:
+		break;
 	case PARSER_CB_ITEM:
 		if (strcmp(path, "uidgid.uid") == 0) {
 			uid = uid_determine(value);
@@ -1628,6 +1809,7 @@ static int read_uidgid_files_into_icmap(
 
 			goto error_exit;
 		}
+		// coverity[TOCTOU:SUPPRESS] not really problem
 		res = stat (filename, &stat_buf);
 		if (res == 0 && S_ISREG(stat_buf.st_mode)) {
 
