@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2002-2006 MontaVista Software, Inc.
- * Copyright (c) 2006-2018 Red Hat, Inc.
+ * Copyright (c) 2006-2021 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -163,12 +163,18 @@ struct sched_param global_sched_param;
 
 static corosync_timer_handle_t corosync_stats_timer_handle;
 
-static const char *corosync_lock_file = LOCALSTATEDIR"/run/corosync.pid";
+static char corosync_lock_file[PATH_MAX + 1] = LOCALSTATEDIR"/run/corosync.pid";
 
 static char corosync_config_file[PATH_MAX + 1] = COROSYSCONFDIR "/corosync.conf";
 
 static int lockfile_fd = -1;
 
+enum move_to_root_cgroup_mode {
+	MOVE_TO_ROOT_CGROUP_MODE_OFF = 0,
+	MOVE_TO_ROOT_CGROUP_MODE_ON = 1,
+	MOVE_TO_ROOT_CGROUP_MODE_AUTO = 2,
+};
+
 qb_loop_t *cs_poll_handle_get (void)
 {
 	return (corosync_poll_handle);
@@ -623,7 +629,7 @@ int main_mcast (
         unsigned int iov_len,
         unsigned int guarantee)
 {
-	const struct qb_ipc_request_header *req = iovec->iov_base;
+	const struct qb_ipc_request_header *req = (struct qb_ipc_request_header *)iovec->iov_base;
 	int32_t service;
 	int32_t fn_id;
 
@@ -859,7 +865,12 @@ static void timer_function_scheduler_timeout (void *data)
 }
 
 
-static int corosync_set_rr_scheduler (void)
+/*
+ * Set main pid RR scheduler.
+ * silent: don't log sched_get_priority_max and sched_setscheduler errors
+ * Returns: 0 - success, -1 failure, -2 platform doesn't support SCHED_RR
+ */
+static int corosync_set_rr_scheduler (int silent)
 {
 	int ret_val = 0;
 
@@ -871,9 +882,11 @@ static int corosync_set_rr_scheduler (void)
 		global_sched_param.sched_priority = sched_priority;
 		res = sched_setscheduler (0, SCHED_RR, &global_sched_param);
 		if (res == -1) {
-			LOGSYS_PERROR(errno, LOGSYS_LEVEL_WARNING,
-				"Could not set SCHED_RR at priority %d",
-				global_sched_param.sched_priority);
+			if (!silent) {
+				LOGSYS_PERROR(errno, LOGSYS_LEVEL_WARNING,
+					"Could not set SCHED_RR at priority %d",
+					global_sched_param.sched_priority);
+			}
 
 			global_sched_param.sched_priority = 0;
 #ifdef HAVE_QB_LOG_THREAD_PRIORITY_SET
@@ -898,15 +911,17 @@ static int corosync_set_rr_scheduler (void)
 			}
 		}
 	} else {
-		LOGSYS_PERROR (errno, LOGSYS_LEVEL_WARNING,
-			"Could not get maximum scheduler priority");
+		if (!silent) {
+			LOGSYS_PERROR (errno, LOGSYS_LEVEL_WARNING,
+				"Could not get maximum scheduler priority");
+		}
 		sched_priority = 0;
 		ret_val = -1;
 	}
 #else
 	log_printf(LOGSYS_LEVEL_WARNING,
 		"The Platform is missing process priority setting features.  Leaving at default.");
-	ret_val = -1;
+	ret_val = -2;
 #endif
 
 	return (ret_val);
@@ -1045,11 +1060,12 @@ static void set_icmap_ro_keys_flag (void)
 	 */
 	icmap_set_ro_access("totem.crypto_cipher", CS_FALSE, CS_TRUE);
 	icmap_set_ro_access("totem.crypto_hash", CS_FALSE, CS_TRUE);
+	icmap_set_ro_access("totem.crypto_model", CS_FALSE, CS_TRUE);
 	icmap_set_ro_access("totem.keyfile", CS_FALSE, CS_TRUE);
 	icmap_set_ro_access("totem.key", CS_FALSE, CS_TRUE);
 	icmap_set_ro_access("totem.secauth", CS_FALSE, CS_TRUE);
 	icmap_set_ro_access("totem.ip_version", CS_FALSE, CS_TRUE);
-	icmap_set_ro_access("totem.rrp_mode", CS_FALSE, CS_TRUE);
+	icmap_set_ro_access("totem.ip_dscp", CS_FALSE, CS_TRUE);
 	icmap_set_ro_access("totem.transport", CS_FALSE, CS_TRUE);
 	icmap_set_ro_access("totem.cluster_name", CS_FALSE, CS_TRUE);
 	icmap_set_ro_access("totem.netmtu", CS_FALSE, CS_TRUE);
@@ -1172,6 +1188,7 @@ error_close:
 static int corosync_move_to_root_cgroup(void) {
 	FILE *f;
 	int res = -1;
+	const char *cgroup_task_fname = NULL;
 
 	/*
 	 * /sys/fs/cgroup is hardcoded, because most of Linux distributions are now
@@ -1182,15 +1199,29 @@ static int corosync_move_to_root_cgroup(void) {
 	 */
 	f = fopen("/sys/fs/cgroup/cpu/cpu.rt_runtime_us", "rt");
 	if (f == NULL) {
-		log_printf(LOGSYS_LEVEL_DEBUG, "cpu.rt_runtime_us doesn't exists -> "
-		    "system without cgroup or with disabled CONFIG_RT_GROUP_SCHED");
+		/*
+		 * Try cgroup v2
+		 */
+		f = fopen("/sys/fs/cgroup/cgroup.procs", "rt");
+		if (f == NULL) {
+			log_printf(LOG_DEBUG, "cpu.rt_runtime_us or cgroup.procs doesn't exist -> "
+			    "system without cgroup or with disabled CONFIG_RT_GROUP_SCHED");
 
-		res = 0;
-		goto exit_res;
+			res = 0;
+			goto exit_res;
+		} else {
+			log_printf(LOGSYS_LEVEL_DEBUG, "Moving main pid to cgroup v2 root cgroup");
+
+			cgroup_task_fname = "/sys/fs/cgroup/cgroup.procs";
+		}
+	} else {
+		log_printf(LOGSYS_LEVEL_DEBUG, "Moving main pid to cgroup v1 root cgroup");
+
+		cgroup_task_fname = "/sys/fs/cgroup/cpu/tasks";
 	}
 	(void)fclose(f);
 
-	f = fopen("/sys/fs/cgroup/cpu/tasks", "w");
+	f = fopen(cgroup_task_fname, "w");
 	if (f == NULL) {
 		log_printf(LOGSYS_LEVEL_WARNING, "Can't open cgroups tasks file for writing");
 
@@ -1214,13 +1245,49 @@ exit_res:
 	 return (res);
 }
 
+static void show_version_info_crypto(void)
+{
+	const char *error_string;
+	const char *list_str;
+
+	if (util_is_valid_knet_crypto_model(NULL, &list_str, 1, "", &error_string) != -1) {
+		printf("Available crypto models: %s\n", list_str);
+	} else {
+		perror(error_string);
+	}
+}
+
+static void show_version_info_compress(void)
+{
+	const char *error_string;
+	const char *list_str;
+
+	if (util_is_valid_knet_compress_model(NULL, &list_str, 1, "", &error_string) != -1) {
+		printf("Available compression models: %s\n", list_str);
+	} else {
+		perror(error_string);
+	}
+}
+
+static void show_version_info(void)
+{
+
+	printf ("Corosync Cluster Engine, version '%s'\n", VERSION);
+	printf ("Copyright (c) 2006-2021 Red Hat, Inc.\n");
+
+	printf ("\nBuilt-in features:" PACKAGE_FEATURES "\n");
+
+	show_version_info_crypto();
+	show_version_info_compress();
+}
 
 int main (int argc, char **argv, char **envp)
 {
 	const char *error_string;
 	struct totem_config totem_config;
 	int res, ch;
-	int background, sched_rr, prio, testonly, move_to_root_cgroup;
+	int background, sched_rr, prio, testonly;
+	enum move_to_root_cgroup_mode move_to_root_cgroup;
 	enum e_corosync_done flock_err;
 	uint64_t totem_config_warnings;
 	struct scheduler_pause_timeout_data scheduler_pause_timeout_data;
@@ -1228,13 +1295,14 @@ int main (int argc, char **argv, char **envp)
 	char *ep;
 	char *tmp_str;
 	int log_subsys_id_totem;
+	int silent;
 
 	/* default configuration
 	 */
 	background = 1;
 	testonly = 0;
 
-	while ((ch = getopt (argc, argv, "c:ftv")) != EOF) {
+	while ((ch = getopt (argc, argv, "c:l:ftv")) != EOF) {
 
 		switch (ch) {
 			case 'c':
@@ -1247,6 +1315,16 @@ int main (int argc, char **argv, char **envp)
 					return EXIT_FAILURE;
 				}
 				break;
+			case 'l':
+				res = snprintf(corosync_lock_file, sizeof(corosync_lock_file), "%s", optarg);
+				if (res >= sizeof(corosync_lock_file)) {
+					fprintf (stderr, "PID lock file path too long.\n");
+					syslog (LOGSYS_LEVEL_ERROR, "PID lock file path too long.");
+
+					logsys_system_fini();
+					return EXIT_FAILURE;
+				}
+				break;
 			case 'f':
 				background = 0;
 				break;
@@ -1254,8 +1332,7 @@ int main (int argc, char **argv, char **envp)
 				testonly = 1;
 				break;
 			case 'v':
-				printf ("Corosync Cluster Engine, version '%s'\n", VERSION);
-				printf ("Copyright (c) 2006-2018 Red Hat, Inc.\n");
+				show_version_info();
 				logsys_system_fini();
 				return EXIT_SUCCESS;
 
@@ -1264,9 +1341,10 @@ int main (int argc, char **argv, char **envp)
 				fprintf(stderr, \
 					"usage:\n"\
 					"        -c     : Corosync config file path.\n"\
+					"        -l     : Corosync pid lock file path.\n"\
 					"        -f     : Start application in foreground.\n"\
 					"        -t     : Test configuration and exit.\n"\
-					"        -v     : Display version and SVN revision of Corosync and exit.\n");
+					"        -v     : Display version, git revision and some useful information about Corosync and exit.\n");
 				logsys_system_fini();
 				return EXIT_FAILURE;
 		}
@@ -1358,8 +1436,9 @@ int main (int argc, char **argv, char **envp)
 		log_printf (LOGSYS_LEVEL_WARNING, "member section is deprecated.");
 	}
 
-	if (totem_config_warnings & TOTEM_CONFIG_WARNING_TOTEM_NODEID_IGNORED) {
-		log_printf (LOGSYS_LEVEL_WARNING, "nodeid appears both in totem section and nodelist. Nodelist one is used.");
+	if (totem_config_warnings & TOTEM_CONFIG_WARNING_TOTEM_NODEID_SET) {
+		log_printf (LOGSYS_LEVEL_WARNING, "nodeid in totem section is deprecated and ignored. "
+		    "Nodelist (or autogenerated) nodeid is going to be used.");
 	}
 
 	if (totem_config_warnings & TOTEM_CONFIG_BINDNETADDR_NODELIST_SET) {
@@ -1382,21 +1461,19 @@ int main (int argc, char **argv, char **envp)
 	}
 
 
-	move_to_root_cgroup = 1;
+	move_to_root_cgroup = MOVE_TO_ROOT_CGROUP_MODE_AUTO;
 	if (icmap_get_string("system.move_to_root_cgroup", &tmp_str) == CS_OK) {
-		if (strcmp(tmp_str, "yes") != 0) {
-			move_to_root_cgroup = 0;
+		/*
+		 * Validity of move_to_root_cgroup values checked in coroparse.c
+		 */
+		if (strcmp(tmp_str, "yes") == 0) {
+			move_to_root_cgroup = MOVE_TO_ROOT_CGROUP_MODE_ON;
+		} else if (strcmp(tmp_str, "no") == 0) {
+			move_to_root_cgroup = MOVE_TO_ROOT_CGROUP_MODE_OFF;
 		}
 		free(tmp_str);
 	}
 
-	/*
-	 * Try to move corosync into root cpu cgroup. Failure is not fatal and
-	 * error is deliberately ignored.
-	 */
-	if (move_to_root_cgroup) {
-		(void)corosync_move_to_root_cgroup();
-	}
 
 	sched_rr = 1;
 	if (icmap_get_string("system.sched_rr", &tmp_str) == CS_OK) {
@@ -1427,11 +1504,31 @@ int main (int argc, char **argv, char **envp)
 		free(tmp_str);
 	}
 
+	if (move_to_root_cgroup == MOVE_TO_ROOT_CGROUP_MODE_ON) {
+		/*
+		 * Try to move corosync into root cpu cgroup. Failure is not fatal and
+		 * error is deliberately ignored.
+		 */
+		(void)corosync_move_to_root_cgroup();
+	}
+
 	/*
 	 * Set round robin realtime scheduling with priority 99
 	 */
 	if (sched_rr) {
-		if (corosync_set_rr_scheduler () != 0) {
+		silent = (move_to_root_cgroup == MOVE_TO_ROOT_CGROUP_MODE_AUTO);
+		res = corosync_set_rr_scheduler (silent);
+
+		if (res == -1 && move_to_root_cgroup == MOVE_TO_ROOT_CGROUP_MODE_AUTO) {
+			/*
+			 * Try to move process to root cgroup and try set priority again
+			 */
+			(void)corosync_move_to_root_cgroup();
+
+			res = corosync_set_rr_scheduler (0);
+		}
+
+		if (res != 0) {
 			prio = INT_MIN;
 		} else {
 			prio = 0;
