@@ -405,6 +405,7 @@ int totem_volatile_config_validate (
 		goto parse_error;
 	}
 
+	// coverity[NO_EFFECT:SUPPRESS] clarify bounds of token_warning values and defensive programming
 	if (totem_config->token_warning > 100 || totem_config->token_warning < 0) {
 		snprintf (local_error_reason, sizeof(local_error_reason),
 			"The token warning parameter (%d%%) must be between 0 (disabled) and 100.",
@@ -1697,14 +1698,21 @@ static int get_interface_params(struct totem_config *totem_config, icmap_map_t m
 			totem_config->interfaces[linknumber].knet_transport = KNET_DEFAULT_TRANSPORT;
 			snprintf(tmp_key, ICMAP_KEYNAME_MAXLEN, "totem.interface.%u.knet_transport", linknumber);
 			if (icmap_get_string_r(map, tmp_key, &str) == CS_OK) {
+#ifdef KNET_TRANSPORT_SCTP
 				if (strcmp(str, "sctp") == 0) {
 					totem_config->interfaces[linknumber].knet_transport = KNET_TRANSPORT_SCTP;
-				}
-				else if (strcmp(str, "udp") == 0) {
+					log_printf(LOGSYS_LEVEL_WARNING, "WARNING SCTP transport is deprecated and will be removed in a future release.\n");
+				} else
+#endif
+				if (strcmp(str, "udp") == 0) {
 					totem_config->interfaces[linknumber].knet_transport = KNET_TRANSPORT_UDP;
 				}
 				else {
+#ifdef KNET_TRANSPORT_SCTP
 					*error_string = "Unrecognised knet_transport. expected 'udp' or 'sctp'";
+#else
+					*error_string = "Unrecognised knet_transport. only 'udp' is supported";
+#endif
 					ret = -1;
 					goto out;
 				}
@@ -1911,6 +1919,8 @@ extern int totem_config_read (
 			       "255.255.255.255", TOTEM_IP_VERSION_4);
 	}
 
+	totem_config->ip_dscp = 0;
+	(void)icmap_get_uint8("totem.ip_dscp", &totem_config->ip_dscp);
 
 	/*
 	 * Store automatically generated items back to icmap only for UDP
@@ -1935,6 +1945,19 @@ extern int totem_config_read (
 		}
 	}
 
+	/*
+	 * Store mcastport value to cmap runtime section for KNET
+	 */
+	if (totem_config->transport_number == TOTEM_TRANSPORT_KNET) {
+		for (i = 0; i < INTERFACE_MAX; i++) {
+			if (!totem_config->interfaces[i].configured) {
+				continue;
+			}
+			snprintf(tmp_key, ICMAP_KEYNAME_MAXLEN, "runtime.config.totem.interface.%u.mcastport", i);
+			icmap_set_uint16(tmp_key, totem_config->interfaces[i].ip_port);
+		}
+	}
+
 	/*
 	 * Check existence of nodelist
 	 */
