@@ -44,7 +44,6 @@
 #include <sys/ioctl.h>
 #include <sys/param.h>
 #include <netinet/in.h>
-#include <net/ethernet.h>
 #include <arpa/inet.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -148,7 +147,7 @@ struct totemknet_instance {
 
 	void *knet_context;
 
-	char iov_buffer[KNET_MAX_PACKET_SIZE];
+	char iov_buffer[KNET_MAX_PACKET_SIZE + 1];
 
 	char *link_status[INTERFACE_MAX];
 
@@ -260,6 +259,27 @@ do {												\
 
 
 #ifdef HAVE_LIBNOZZLE
+static inline uint8_t *ether_get_dst_mac(const unsigned char *packet, ssize_t packet_len)
+{
+	/*
+	 * Packet must be at least as long as one MAC address
+	 */
+	if (packet_len < 6) {
+		return NULL;
+	}
+
+	/*
+	 * To ensure portability, this implementation parses raw packet data directly
+	 * according to the Ethernet II standard.
+	 *
+	 * The system-provided struct ether_header is not used because its definition
+	 * and the location of its header file vary between platforms like Linux, BSD,
+	 * and Illumos. In the Ethernet II standard, the destination MAC address is
+	 * always the first 6 bytes of the frame.
+	 */
+	return (uint8_t *)packet;
+}
+
 static inline int is_ether_addr_multicast(const uint8_t *addr)
 {
 	return (addr[0] & 0x01);
@@ -279,10 +299,13 @@ static int ether_host_filter_fn(void *private_data,
 				knet_node_id_t *dst_host_ids,
 				size_t *dst_host_ids_entries)
 {
-	struct ether_header *eth_h = (struct ether_header *)outdata;
-	uint8_t *dst_mac = (uint8_t *)eth_h->ether_dhost;
+	uint8_t *dst_mac = ether_get_dst_mac(outdata, outdata_len);
 	uint16_t dst_host_id;
 
+	if (dst_mac == NULL) {
+		return -1;
+	}
+
 	if (is_ether_addr_zero(dst_mac))
 		return -1;
 
@@ -393,21 +416,6 @@ static inline void ucast_sendmsg (
 	memset(&msg_ucast, 0, sizeof(msg_ucast));
 	msg_ucast.msg_iov = (void *)&iovec;
 	msg_ucast.msg_iovlen = 1;
-#ifdef HAVE_MSGHDR_CONTROL
-	msg_ucast.msg_control = 0;
-#endif
-#ifdef HAVE_MSGHDR_CONTROLLEN
-	msg_ucast.msg_controllen = 0;
-#endif
-#ifdef HAVE_MSGHDR_FLAGS
-	msg_ucast.msg_flags = 0;
-#endif
-#ifdef HAVE_MSGHDR_ACCRIGHTS
-	msg_ucast.msg_accrights = NULL;
-#endif
-#ifdef HAVE_MSGHDR_ACCRIGHTSLEN
-	msg_ucast.msg_accrightslen = 0;
-#endif
 
 	/*
 	 * Transmit unicast message
@@ -443,22 +451,6 @@ static inline void mcast_sendmsg (
 	memset(&msg_mcast, 0, sizeof(msg_mcast));
 	msg_mcast.msg_iov = (void *)&iovec;
 	msg_mcast.msg_iovlen = 1;
-#ifdef HAVE_MSGHDR_CONTROL
-	msg_mcast.msg_control = 0;
-#endif
-#ifdef HAVE_MSGHDR_CONTROLLEN
-	msg_mcast.msg_controllen = 0;
-#endif
-#ifdef HAVE_MSGHDR_FLAGS
-	msg_mcast.msg_flags = 0;
-#endif
-#ifdef HAVE_MSGHDR_ACCRIGHTS
-	msg_mcast.msg_accrights = NULL;
-#endif
-#ifdef HAVE_MSGHDR_ACCRIGHTSLEN
-	msg_mcast.msg_accrightslen = 0;
-#endif
-
 
 //	log_printf (LOGSYS_LEVEL_DEBUG, "totemknet: mcast_sendmsg. only_active=%d, len=%d", only_active, msg_len);
 
@@ -795,67 +787,56 @@ static int data_deliver_fn (
 	struct iovec iov_recv;
 	struct sockaddr_storage system_from;
 	ssize_t msg_len;
-	int truncated_packet;
+	char *data_ptr = instance->iov_buffer;
 
 	iov_recv.iov_base = instance->iov_buffer;
-	iov_recv.iov_len = KNET_MAX_PACKET_SIZE;
+	iov_recv.iov_len = KNET_MAX_PACKET_SIZE + 1;
 
+	memset(&msg_hdr, 0, sizeof(msg_hdr));
 	msg_hdr.msg_name = &system_from;
 	msg_hdr.msg_namelen = sizeof (struct sockaddr_storage);
 	msg_hdr.msg_iov = &iov_recv;
 	msg_hdr.msg_iovlen = 1;
-#ifdef HAVE_MSGHDR_CONTROL
-	msg_hdr.msg_control = 0;
-#endif
-#ifdef HAVE_MSGHDR_CONTROLLEN
-	msg_hdr.msg_controllen = 0;
-#endif
-#ifdef HAVE_MSGHDR_FLAGS
-	msg_hdr.msg_flags = 0;
-#endif
-#ifdef HAVE_MSGHDR_ACCRIGHTS
-	msg_hdr.msg_accrights = NULL;
-#endif
-#ifdef HAVE_MSGHDR_ACCRIGHTSLEN
-	msg_hdr.msg_accrightslen = 0;
-#endif
 
 	msg_len = recvmsg (fd, &msg_hdr, MSG_NOSIGNAL | MSG_DONTWAIT);
 	if (msg_len <= 0) {
 		return (0);
 	}
 
-	truncated_packet = 0;
-
-#ifdef HAVE_MSGHDR_FLAGS
-	if (msg_hdr.msg_flags & MSG_TRUNC) {
-		truncated_packet = 1;
+	if (msg_len >= KNET_MAX_PACKET_SIZE + 1) {
+		/*
+		 * It this happens it is real bug, because knet always sends packet with maximum size
+		 * of KNET_MAX_PACKET_SIZE.
+		 * If received packet is MAX_PACKET_SIZE + 1 it means packet was truncated
+		 * (iov_buffer size and iov_len are intentionally set to KNET_MAX_PACKET_SIZE + 1).
+		 */
+		knet_log_printf(instance->totemknet_log_level_error,
+				"Received truncated packet. Please report this bug. Dropping packet.");
+		return (0);
 	}
-#else
+
 	/*
-	 * We don't have MSGHDR_FLAGS, but we can (hopefully) safely make assumption that
-	 * if bytes_received == KNET_MAX_PACKET_SIZE then packet is truncated
+	 * If it's from the knet fd then it will have the optional knet header on it
 	 */
-	if (bytes_received == KNET_MAX_PACKET_SIZE) {
-		truncated_packet = 1;
+#ifdef KNET_DATAFD_FLAG_RX_RETURN_INFO
+	if (fd == instance->knet_fd) {
+		struct knet_datafd_header *datafd_header = (struct knet_datafd_header *)data_ptr;
+
+/* 		knet_log_printf (LOGSYS_LEVEL_DEBUG, "Packet from knet_fd nodeid: %d\n", datafd_header->src_nodeid); */
+
+		/* Advance past the ACTUAL header size, not the size we think it might be */
+		data_ptr += datafd_header->size;
+		msg_len -= datafd_header->size;
 	}
 #endif
 
-	if (truncated_packet) {
-		knet_log_printf(instance->totemknet_log_level_error,
-				"Received too big message. This may be because something bad is happening"
-				"on the network (attack?), or you tried join more nodes than corosync is"
-				"compiled with (%u) or bug in the code (bad estimation of "
-				"the KNET_MAX_PACKET_SIZE). Dropping packet.", PROCESSOR_COUNT_MAX);
-		return (0);
-	}
 
 	/*
 	 * Handle incoming message
 	 */
 	instance->totemknet_deliver_fn (
 		instance->context,
-		instance->iov_buffer,
+		data_ptr,
 		msg_len,
 		&system_from);
 
@@ -900,7 +881,7 @@ static void knet_set_access_list_config(struct totemknet_instance *instance)
 #endif
 }
 
-void totemknet_configure_log_level()
+void totemknet_configure_log_level(void)
 {
 	int logsys_log_mode;
 	int knet_log_mode = KNET_LOG_INFO;
@@ -1279,6 +1260,12 @@ int totemknet_initialize (
 	if (res) {
 		KNET_LOGSYS_PERROR(errno, LOGSYS_LEVEL_WARNING, "knet_handle_pmtud_set failed");
 	}
+#ifdef HAVE_KNET_SETPRIO_DSCP
+	res = knet_handle_setprio_dscp(instance->knet_handle, instance->totem_config->ip_dscp);
+	if (res) {
+		KNET_LOGSYS_PERROR(errno, LOGSYS_LEVEL_WARNING, "knet_handle_setprio_dscp failed");
+	}
+#endif
 	res = knet_handle_enable_filter(instance->knet_handle, instance, dst_host_filter_callback_fn);
 	if (res) {
 		KNET_LOGSYS_PERROR(errno, LOGSYS_LEVEL_WARNING, "knet_handle_enable_filter failed");
@@ -1302,7 +1289,11 @@ int totemknet_initialize (
 
 	/* Get an fd into knet */
 	instance->knet_fd = 0;
+#ifdef KNET_DATAFD_FLAG_RX_RETURN_INFO
+	res = knet_handle_add_datafd(instance->knet_handle, &instance->knet_fd, &channel, KNET_DATAFD_FLAG_RX_RETURN_INFO);
+#else
 	res = knet_handle_add_datafd(instance->knet_handle, &instance->knet_fd, &channel);
+#endif
 	if (res) {
 		knet_log_printf(LOG_DEBUG, "knet_handle_add_datafd failed: %s", strerror(errno));
 		goto exit_error;
@@ -1521,25 +1512,11 @@ extern int totemknet_recv_mcast_empty (
 	iov_recv.iov_base = instance->iov_buffer;
 	iov_recv.iov_len = KNET_MAX_PACKET_SIZE;
 
+	memset(&msg_hdr, 0, sizeof(msg_hdr));
 	msg_hdr.msg_name = &system_from;
 	msg_hdr.msg_namelen = sizeof (struct sockaddr_storage);
 	msg_hdr.msg_iov = &iov_recv;
 	msg_hdr.msg_iovlen = 1;
-#ifdef HAVE_MSGHDR_CONTROL
-	msg_hdr.msg_control = 0;
-#endif
-#ifdef HAVE_MSGHDR_CONTROLLEN
-	msg_hdr.msg_controllen = 0;
-#endif
-#ifdef HAVE_MSGHDR_FLAGS
-	msg_hdr.msg_flags = 0;
-#endif
-#ifdef HAVE_MSGHDR_ACCRIGHTS
-	msg_msg_hdr.msg_accrights = NULL;
-#endif
-#ifdef HAVE_MSGHDR_ACCRIGHTSLEN
-	msg_msg_hdr.msg_accrightslen = 0;
-#endif
 
 	do {
 		ufd.fd = instance->knet_fd;
@@ -2166,7 +2143,11 @@ static int create_nozzle_device(void *knet_context, const char *name,
 	nozzle_fd = nozzle_get_fd(nozzle_dev);
 	knet_log_printf (LOGSYS_LEVEL_INFO, "Opened '%s' on fd %d", device_name, nozzle_fd);
 
+#ifdef KNET_DATAFD_FLAG_RX_RETURN_INFO
+	res = knet_handle_add_datafd(instance->knet_handle, &nozzle_fd, &channel, 0);
+#else
 	res = knet_handle_add_datafd(instance->knet_handle, &nozzle_fd, &channel);
+#endif
 	if (res != 0) {
 		knet_log_printf (LOGSYS_LEVEL_ERROR, "Unable to add nozzle FD to knet: %s", strerror(errno));
 		goto out_clean;
