@@ -424,7 +424,7 @@ static inline void group_endian_convert (
 	/*
 	 * Align data structure for not i386 or x86_64
 	 */
-	if ((size_t)msg % 4 != 0) {
+	if ((size_t)msg % sizeof(char *) != 0) {
 		aligned_msg = alloca(msg_len);
 		memcpy(aligned_msg, msg, msg_len);
 	} else {
@@ -466,7 +466,7 @@ static inline int group_matches (
 	/*
 	 * Align data structure for not i386 or x86_64
 	 */
-	if ((size_t)iovec->iov_base % 4 != 0) {
+	if ((size_t)iovec->iov_base % sizeof(char *) != 0) {
 		iovec_aligned.iov_base = alloca(iovec->iov_len);
 		memcpy(iovec_aligned.iov_base, iovec->iov_base, iovec->iov_len);
 		iovec_aligned.iov_len = iovec->iov_len;
@@ -551,7 +551,7 @@ static inline void app_deliver_fn (
 			/*
 			 * Align data structure for not i386 or x86_64
 			 */
-			if ((char *)iovec->iov_base + adjust_iovec % 4 != 0) {
+			if ((uintptr_t)((char *)iovec->iov_base + adjust_iovec) % (sizeof(char *)) != 0) {
 				/*
 				 * Deal with misalignment
 				 */
@@ -857,6 +857,7 @@ void totempg_finalize (void)
 	if (totempg_threaded_mode == 1) {
 		pthread_mutex_lock (&totempg_mutex);
 	}
+	// coverity[SLEEP:SUPPRESS] sleep is not a problem because it is shutdown
 	totemsrp_finalize (totemsrp_context);
 	if (totempg_threaded_mode == 1) {
 		pthread_mutex_unlock (&totempg_mutex);
@@ -942,6 +943,7 @@ static int mcast_msg (
 			fragment_size += copy_len;
 			mcast_packed_msg_lens[mcast_packed_msg_count] += copy_len;
 			next_fragment = 1;
+			// coverity[UNUSED_VALUE:SUPPRESS] defensive programming
 			copy_len = 0;
 			copy_base = 0;
 			i++;
@@ -1011,6 +1013,7 @@ static int mcast_msg (
 			 * If the iovec all fit, go to the next iovec
 			 */
 			if ((copy_base + copy_len) == iovec[i].iov_len) {
+				// coverity[UNUSED_VALUE:SUPPRESS] defensive programming
 				copy_len = 0;
 				copy_base = 0;
 				i++;
@@ -1103,7 +1106,7 @@ int totempg_callback_token_create (
 	int (*callback_fn) (enum totem_callback_token_type type, const void *),
 	const void *data)
 {
-	unsigned int res;
+	int res;
 	if (totempg_threaded_mode == 1) {
 		pthread_mutex_lock (&callback_token_mutex);
 	}
@@ -1239,7 +1242,7 @@ int totempg_groups_mcast_joined (
 	unsigned short group_len[MAX_GROUPS_PER_MSG + 1];
 	struct iovec iovec_mcast[MAX_GROUPS_PER_MSG + 1 + MAX_IOVECS_FROM_APP];
 	int i;
-	unsigned int res;
+	int res;
 
 	if (totempg_threaded_mode == 1) {
 		pthread_mutex_lock (&totempg_mutex);
@@ -1255,7 +1258,7 @@ int totempg_groups_mcast_joined (
 		iovec_mcast[i + 1].iov_base = (void *) instance->groups[i].group;
 	}
 	iovec_mcast[0].iov_len = (instance->groups_cnt + 1) * sizeof (unsigned short);
-	iovec_mcast[0].iov_base = group_len;
+	iovec_mcast[0].iov_base = (char *)group_len;
 	for (i = 0; i < iov_len; i++) {
 		iovec_mcast[i + instance->groups_cnt + 1].iov_len = iovec[i].iov_len;
 		iovec_mcast[i + instance->groups_cnt + 1].iov_base = iovec[i].iov_base;
@@ -1368,7 +1371,7 @@ int totempg_groups_mcast_groups (
 	unsigned short group_len[MAX_GROUPS_PER_MSG + 1];
 	struct iovec iovec_mcast[MAX_GROUPS_PER_MSG + 1 + MAX_IOVECS_FROM_APP];
 	int i;
-	unsigned int res;
+	int res;
 
 	if (totempg_threaded_mode == 1) {
 		pthread_mutex_lock (&totempg_mutex);
@@ -1384,7 +1387,7 @@ int totempg_groups_mcast_groups (
 		iovec_mcast[i + 1].iov_base = (void *) groups[i].group;
 	}
 	iovec_mcast[0].iov_len = (groups_cnt + 1) * sizeof (unsigned short);
-	iovec_mcast[0].iov_base = group_len;
+	iovec_mcast[0].iov_base = (char *)group_len;
 	for (i = 0; i < iov_len; i++) {
 		iovec_mcast[i + groups_cnt + 1].iov_len = iovec[i].iov_len;
 		iovec_mcast[i + groups_cnt + 1].iov_base = iovec[i].iov_base;
@@ -1447,6 +1450,13 @@ int totempg_iface_set (
 	return (res);
 }
 
+int totempg_nodestatus_get (unsigned int nodeid,
+			    struct totem_node_status *node_status)
+{
+	memset(node_status, 0, sizeof(struct totem_node_status));
+	return totemsrp_nodestatus_get (totemsrp_context, nodeid, node_status);
+}
+
 int totempg_ifaces_get (
 	unsigned int nodeid,
 	unsigned int *interface_id,
