@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2011 Red Hat, Inc.
+ * Copyright (c) 2009-2025 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -38,9 +38,11 @@
 
 #include <config.h>
 
+#include <sys/types.h>
+#include <sys/resource.h>
+
 #include <limits.h>
 #include <pthread.h>
-#include <sys/types.h>
 #include <stdio.h>
 #include <stdint.h>
 #include <stdlib.h>
@@ -51,11 +53,9 @@
 #include <string.h>
 #include <sys/wait.h>
 #include <corosync/cmap.h>
-extern const char *__progname;
 
-static int test2_sig_delivered = 0;
-static int test5_hc_cb_count = 0;
-static int test6_sig_delivered = 0;
+static int test_sig_delivered = 0;
+static int test_hc_cb_count = 0;
 
 /*
  * First test will just register SAM, with policy restart. First instance will
@@ -67,7 +67,7 @@ static int test6_sig_delivered = 0;
  * twice. One should succeed, second should fail. After this, we will call every function
  * (none should succeed).
  */
-static int test1 (void)
+static int test_basics (void)
 {
 	cs_error_t error;
 	unsigned int instance_id;
@@ -111,7 +111,7 @@ static int test1 (void)
 			error = sam_stop ();
 
 			if (error != CS_OK) {
-				fprintf (stderr, "Can't send hc. Error %d\n", error);
+				fprintf (stderr, "Can't stop hc. Error %d\n", error);
 				return 1;
 			}
 		}
@@ -119,6 +119,12 @@ static int test1 (void)
 		printf ("%s iid %d: sleep 3\n", __FUNCTION__, instance_id);
 		sleep (3);
 
+		if (instance_id == 1) {
+			printf ("%s iid %d: Failed. Wasn't killed.\n", __FUNCTION__, instance_id);
+
+			return 1;
+		}
+
 		printf ("%s iid %d: start\n", __FUNCTION__, instance_id);
 		error = sam_start ();
 		if (error != CS_OK) {
@@ -176,19 +182,21 @@ static int test1 (void)
 }
 
 
-static void test2_signal (int sig) {
+static void test_signal_handler (int sig) {
 	printf ("%s\n", __FUNCTION__);
 
-	test2_sig_delivered = 1;
+	test_sig_delivered = 1;
 }
 
 /*
  * This tests recovery policy quit and callback.
  */
-static int test2 (void) {
+static int test_signal (void) {
 	cs_error_t error;
 	unsigned int instance_id;
 
+	test_sig_delivered = 0;
+
 	printf ("%s: initialize\n", __FUNCTION__);
 	error = sam_initialize (2000, SAM_RECOVERY_POLICY_QUIT);
 	if (error != CS_OK) {
@@ -203,7 +211,7 @@ static int test2 (void) {
 	}
 
 	if (instance_id == 1) {
-		signal (SIGTERM, test2_signal);
+		signal (SIGTERM, test_signal_handler);
 
 		printf ("%s iid %d: start\n", __FUNCTION__, instance_id);
 		error = sam_start ();
@@ -224,7 +232,7 @@ static int test2 (void) {
 
 
 		printf ("%s iid %d: wait for delivery of signal\n", __FUNCTION__, instance_id);
-		while (!test2_sig_delivered) {
+		while (!test_sig_delivered) {
 			sleep (1);
 		}
 
@@ -234,16 +242,24 @@ static int test2 (void) {
 	}
 
 	return 1;
-
 }
 
+
 /*
- * Smoke test. Better to turn off coredump ;) This has no time limit, just restart process
+ * Smoke test. This has no time limit, just restart process
  * when it dies.
  */
-static int test3 (void) {
+static int test_smoke (void) {
 	cs_error_t error;
 	unsigned int instance_id;
+	struct rlimit lim;
+
+	lim.rlim_cur = lim.rlim_max = 0;
+
+	/*
+	 * Try to turn off core creation
+	 */
+	setrlimit(RLIMIT_CORE, &lim);
 
 	printf ("%s: initialize\n", __FUNCTION__);
 	error = sam_initialize (0, SAM_RECOVERY_POLICY_RESTART);
@@ -278,7 +294,7 @@ static int test3 (void) {
 /*
  * Test sam_data_store, sam_data_restore and sam_data_getsize
  */
-static int test4 (void)
+static int test_data_store_restore (void)
 {
 	size_t size;
 	cs_error_t err;
@@ -524,16 +540,16 @@ static int test4 (void)
 	return (0);
 }
 
-static int test5_hc_cb (void)
+static int test_hc_cb (void)
 {
 	cs_error_t res;
-	printf ("%s %d\n", __FUNCTION__, ++test5_hc_cb_count);
+	printf ("%s %d\n", __FUNCTION__, ++test_hc_cb_count);
 
-	res = sam_data_store (&test5_hc_cb_count, sizeof (test5_hc_cb_count));
+	res = sam_data_store (&test_hc_cb_count, sizeof (test_hc_cb_count));
 	if (res != CS_OK)
 		return 1;
 
-	if (test5_hc_cb_count > 10)
+	if (test_hc_cb_count > 10)
 		return 1;
 
 	return 0;
@@ -541,7 +557,7 @@ static int test5_hc_cb (void)
 /*
  * Test event driven healtchecking.
  */
-static int test5 (void)
+static int test_hc (void)
 {
 	cs_error_t error;
 	unsigned int instance_id;
@@ -562,7 +578,7 @@ static int test5 (void)
 
 	if (instance_id == 1) {
 		printf ("%s iid %d: hc callback register\n", __FUNCTION__, instance_id);
-		error = sam_hc_callback_register (test5_hc_cb);
+		error = sam_hc_callback_register (test_hc_cb);
 		if (error != CS_OK) {
 			fprintf (stderr, "Can't register hc cb. Error %d\n", error);
 			return 1;
@@ -601,13 +617,13 @@ static int test5 (void)
 	return 1;
 }
 
-static void test6_signal (int sig) {
+static void test_warn_signal_handler (int sig) {
 	cs_error_t error;
 
 	printf ("%s\n", __FUNCTION__);
-	test6_sig_delivered++;
+	test_sig_delivered++;
 
-	if ((error = sam_data_store (&test6_sig_delivered, sizeof (test6_sig_delivered))) != CS_OK) {
+	if ((error = sam_data_store (&test_sig_delivered, sizeof (test_sig_delivered))) != CS_OK) {
 		fprintf (stderr, "Can't store data! Error : %d\n", error);
 	}
 }
@@ -615,10 +631,12 @@ static void test6_signal (int sig) {
 /*
  * Test warn signal set.
  */
-static int test6 (void) {
+static int test_warn_signal (void) {
 	cs_error_t error;
 	unsigned int instance_id;
-	int test6_sig_del;
+	int tmp_sig_delivered;
+
+	test_sig_delivered = 0;
 
 	printf ("%s: initialize\n", __FUNCTION__);
 	error = sam_initialize (2000, SAM_RECOVERY_POLICY_RESTART);
@@ -640,7 +658,7 @@ static int test6 (void) {
 			return 1;
 		}
 
-		signal (SIGUSR1, test6_signal);
+		signal (SIGUSR1, test_warn_signal_handler);
 
 		printf ("%s iid %d: start\n", __FUNCTION__, instance_id);
 		error = sam_start ();
@@ -661,7 +679,7 @@ static int test6 (void) {
 
 
 		printf ("%s iid %d: wait for delivery of signal\n", __FUNCTION__, instance_id);
-		while (!test6_sig_delivered) {
+		while (!test_sig_delivered) {
 			sleep (1);
 		}
 
@@ -674,13 +692,13 @@ static int test6 (void) {
 	}
 
 	if (instance_id == 2) {
-		error = sam_data_restore (&test6_sig_del, sizeof (test6_sig_del));
+		error = sam_data_restore (&tmp_sig_delivered, sizeof (tmp_sig_delivered));
 		if (error != CS_OK) {
 			fprintf (stderr, "Can't restore data. Error %d\n", error);
 			return 1;
 		}
 
-		if (test6_sig_del != 1) {
+		if (tmp_sig_delivered != 1) {
 			fprintf (stderr, "Previous test failed. Signal was not delivered\n");
 			return 1;
 		}
@@ -691,7 +709,7 @@ static int test6 (void) {
 			return 1;
 		}
 
-		signal (SIGUSR1, test6_signal);
+		signal (SIGUSR1, test_warn_signal_handler);
 
 		printf ("%s iid %d: start\n", __FUNCTION__, instance_id);
 		error = sam_start ();
@@ -712,7 +730,7 @@ static int test6 (void) {
 
 
 		printf ("%s iid %d: wait for delivery of signal\n", __FUNCTION__, instance_id);
-		while (!test6_sig_delivered) {
+		while (!test_sig_delivered) {
 			sleep (1);
 		}
 
@@ -721,13 +739,13 @@ static int test6 (void) {
 	}
 
 	if (instance_id == 3) {
-		error = sam_data_restore (&test6_sig_del, sizeof (test6_sig_del));
+		error = sam_data_restore (&tmp_sig_delivered, sizeof (tmp_sig_delivered));
 		if (error != CS_OK) {
 			fprintf (stderr, "Can't restore data. Error %d\n", error);
 			return 1;
 		}
 
-		if (test6_sig_del != 1) {
+		if (tmp_sig_delivered != 1) {
 			fprintf (stderr, "Previous test failed. Signal WAS delivered\n");
 			return 1;
 		}
@@ -738,7 +756,7 @@ static int test6 (void) {
 	return 1;
 }
 
-static void *test7_thread (void *arg)
+static void *test_thread_sleep_exit (void *arg)
 {
 	/* Wait 5s */
 	sleep (5);
@@ -748,12 +766,13 @@ static void *test7_thread (void *arg)
 /*
  * Test quorum
  */
-static int test7 (void) {
+static int test_quorum (void) {
 	cmap_handle_t cmap_handle;
 	cs_error_t err;
 	unsigned int instance_id;
 	pthread_t kill_thread;
 	char *str;
+	uint32_t expected_votes;
 
 	err = cmap_initialize (&cmap_handle);
 	if (err != CS_OK) {
@@ -766,17 +785,27 @@ static int test7 (void) {
 		printf ("Could not get \"provider\" key: %d. Test skipped\n", err);
 		return (1);
 	}
-        if (strcmp(str, "testquorum") != 0) {
-		printf ("Provider is not testquorum. Test skipped\n");
+        if (strcmp(str, "corosync_votequorum") != 0) {
+		printf ("Provider is not corosync_votequorum. Test skipped\n");
 		free(str);
 		return (1);
         }
 	free(str);
 
+	if (cmap_get_uint32(cmap_handle, "quorum.expected_votes", &expected_votes) != CS_OK) {
+		printf ("Could not get \"expected_votes\" key: %d. Test skipped\n", err);
+		return (1);
+	}
+
+	if (expected_votes != 1) {
+		printf ("Expected_votes is not 1. Test skipped\n");
+		return (1);
+	}
+
 	/*
 	 * Set to not quorate
 	 */
-	err = cmap_set_uint8(cmap_handle, "quorum.quorate", 0);
+	err = cmap_set_uint32(cmap_handle, "quorum.expected_votes", 2);
 	if (err != CS_OK) {
 		printf ("Can't set map key. Error %d\n", err);
 		return (2);
@@ -798,9 +827,9 @@ static int test7 (void) {
 
 	if (instance_id == 1) {
 		/*
-		 * Sam start should block forever, but 10s for us should be enough
+		 * Sam start should block forever, but 5s for us should be enough
 		 */
-		pthread_create (&kill_thread, NULL, test7_thread, NULL);
+		pthread_create (&kill_thread, NULL, test_thread_sleep_exit, NULL);
 
 		printf ("%s iid %d: start - should block forever (waiting 5s)\n", __FUNCTION__, instance_id);
 		err = sam_start ();
@@ -817,7 +846,7 @@ static int test7 (void) {
 		/*
 		 * Set to quorate
 		 */
-		err = cmap_set_uint8(cmap_handle, "quorum.quorate", 1);
+		err = cmap_set_uint32(cmap_handle, "quorum.expected_votes", 1);
 		if (err != CS_OK) {
 			printf ("Can't set map key. Error %d\n", err);
 			return (2);
@@ -833,7 +862,7 @@ static int test7 (void) {
 		/*
 		 * Set corosync unquorate
 		 */
-		err = cmap_set_uint8(cmap_handle, "quorum.quorate", 0);
+		err = cmap_set_uint32(cmap_handle, "quorum.expected_votes", 2);
 		if (err != CS_OK) {
 			printf ("Can't set map key. Error %d\n", err);
 			return (2);
@@ -853,10 +882,62 @@ static int test7 (void) {
 	return (2);
 }
 
+static int test_finalize_deadlock (void) {
+	cs_error_t error;
+	unsigned int instance_id;
+	pthread_t kill_thread;
+
+	test_sig_delivered = 0;
+
+	printf ("%s: initialize\n", __FUNCTION__);
+	error = sam_initialize (2000, SAM_RECOVERY_POLICY_RESTART);
+	if (error != CS_OK) {
+		fprintf (stderr, "Can't initialize SAM API. Error %d\n", error);
+		return 1;
+	}
+	printf ("%s: register\n", __FUNCTION__);
+	error = sam_register (&instance_id);
+	if (error != CS_OK) {
+		fprintf (stderr, "Can't register. Error %d\n", error);
+		return 1;
+	}
+
+	if (instance_id == 1) {
+		/*
+		 * Sam finalize might block forever, but 5s for us should be enough
+		 */
+		pthread_create (&kill_thread, NULL, test_thread_sleep_exit, NULL);
+
+		printf ("%s iid %d: start\n", __FUNCTION__, instance_id);
+		error = sam_start ();
+		if (error != CS_OK) {
+			fprintf (stderr, "Can't start hc. Error %d\n", error);
+			return 1;
+		}
+
+		printf ("%s iid %d: call finalize (shouldn't deadlock)\n", __FUNCTION__, instance_id);
+		error = sam_finalize ();
+		if (error != CS_OK) {
+			fprintf (stderr, "Can't finalize sam. Error %d\n", error);
+			return 1;
+		}
+
+		printf ("%s iid %d: finalize succeeded\n", __FUNCTION__, instance_id);
+		return 0;
+	}
+
+	if (instance_id == 2) {
+		printf ("%s iid %d: previous sam_finalize deadlocked\n", __FUNCTION__, instance_id);
+	}
+
+	return 1;
+}
+
 /*
  * Test cmap integration + quit policy
  */
-static int test8 (pid_t pid, pid_t old_pid, int test_n) {
+static int test_cmap_quit (pid_t pid, pid_t old_pid, int test_n)
+{
 	cmap_handle_t cmap_handle;
 	cs_error_t err;
 	uint64_t tstamp1, tstamp2;
@@ -1079,7 +1160,7 @@ static int test8 (pid_t pid, pid_t old_pid, int test_n) {
 		 * Previous should be FAILED
 		 */
 
-		snprintf(key_name, CMAP_KEYNAME_MAXLEN, "resources.process.%d.state", pid);
+		snprintf(key_name, CMAP_KEYNAME_MAXLEN, "resources.process.%d.state", old_pid);
 		err = cmap_get_string(cmap_handle, key_name, &str);
 		if (err != CS_OK) {
 			printf ("Could not get \"state\" key: %d.\n", err);
@@ -1102,7 +1183,7 @@ static int test8 (pid_t pid, pid_t old_pid, int test_n) {
 /*
  * Test cmap integration + restart policy
  */
-static int test9 (pid_t pid, pid_t old_pid, int test_n) {
+static int test_cmap_restart (pid_t pid, pid_t old_pid, int test_n) {
 	cs_error_t err;
 	cmap_handle_t cmap_handle;
 	unsigned int instance_id;
@@ -1211,7 +1292,7 @@ static int test9 (pid_t pid, pid_t old_pid, int test_n) {
 		/*
 		 * Previous should be FAILED
 		 */
-		snprintf(key_name, CMAP_KEYNAME_MAXLEN, "resources.process.%d.state", pid);
+		snprintf(key_name, CMAP_KEYNAME_MAXLEN, "resources.process.%d.state", old_pid);
 		err = cmap_get_string(cmap_handle, key_name, &str);
 		if (err != CS_OK) {
 			printf ("Could not get \"state\" key: %d.\n", err);
@@ -1231,243 +1312,132 @@ static int test9 (pid_t pid, pid_t old_pid, int test_n) {
 	return (2);
 }
 
-int main(int argc, char *argv[])
+static int run_test(const char *test_name, int (test_func)(void), int skippable, int *all_passed, int *no_skipped)
+{
+	pid_t pid;
+	int err;
+	int stat;
+	const char *res_str;
+
+	fprintf (stderr, "Running %s test\n", test_name);
+	pid = fork ();
+
+	if (pid == -1) {
+		fprintf (stderr, "Can't fork\n");
+
+		exit(1);
+	}
+
+	if (pid == 0) {
+		/*
+		 * Child runs tests and sets exit code to result of test
+		 */
+		err = test_func ();
+
+		/*
+		 * After sam_register this is child of child
+		 */
+		sam_finalize ();
+
+		exit (err);
+	}
+
+	/*
+	 * Parent waits for child to run the test and return exit code
+	 */
+	waitpid (pid, &stat, 0);
+
+	err = WEXITSTATUS (stat);
+
+	if (!skippable) {
+		res_str = (err == 0 ? "passed" : "failed");
+
+		if (err != 0) {
+			(*all_passed) = 0;
+		}
+	} else {
+		res_str = (err == 0 ? "passed" : (err == 1 ? "skipped" : "failed"));
+
+		if (err == 1) {
+			(*no_skipped)++;
+		} else if (err > 1) {
+			(*all_passed) = 0;
+		}
+	}
+
+	fprintf (stderr, "%s test %s\n", test_name, res_str);
+
+	return (err);
+}
+
+static int run_cmap_test(const char *test_name,
+    int (test_func)(pid_t pid, pid_t old_pid, int test_n),
+    int no_tests, int *all_passed, int *no_skipped)
 {
 	pid_t pid, old_pid;
 	int err;
 	int stat;
-	int all_passed = 1;
-	int no_skipped = 0;
+	const char *res_str;
+	int n;
 
-	pid = fork ();
+	old_pid = 0;
 
-	if (pid == -1) {
-		fprintf (stderr, "Can't fork\n");
-		return 1;
-	}
+	fprintf (stderr, "Running %s test\n", test_name);
 
-	if (pid == 0) {
-		err = test1 ();
-		sam_finalize ();
-		return err;
-	}
-
-	waitpid (pid, &stat, 0);
-
-	fprintf (stderr, "test1 %s\n", (WEXITSTATUS (stat) == 0 ? "passed" : "failed"));
-	if (WEXITSTATUS (stat) != 0)
-		all_passed = 0;
-
-	pid = fork ();
-
-	if (pid == -1) {
-		fprintf (stderr, "Can't fork\n");
-		return 1;
-	}
-
-	if (pid == 0) {
-		err = test2 ();
-
-		sam_finalize ();
-		return (err);
-	}
-
-	waitpid (pid, &stat, 0);
-
-	fprintf (stderr, "test2 %s\n", (WEXITSTATUS (stat) == 0 ? "passed" : "failed"));
-	if (WEXITSTATUS (stat) != 0)
-		all_passed = 0;
-
-	pid = fork ();
-
-	if (pid == -1) {
-		fprintf (stderr, "Can't fork\n");
-		return 1;
-	}
-
-	if (pid == 0) {
-		err = test3 ();
-		sam_finalize ();
-		return (err);
-	}
-
-	waitpid (pid, &stat, 0);
-
-	fprintf (stderr, "test3 %s\n", (WEXITSTATUS (stat) == 0 ? "passed" : "failed"));
-	if (WEXITSTATUS (stat) != 0)
-		all_passed = 0;
-
-	pid = fork ();
-
-	if (pid == -1) {
-		fprintf (stderr, "Can't fork\n");
-		return 1;
-	}
-
-	if (pid == 0) {
-		err = test4 ();
-		sam_finalize ();
-		return (err);
-	}
-
-	waitpid (pid, &stat, 0);
-
-	fprintf (stderr, "test4 %s\n", (WEXITSTATUS (stat) == 0 ? "passed" : "failed"));
-	if (WEXITSTATUS (stat) != 0)
-		all_passed = 0;
-
-	pid = fork ();
-
-	if (pid == -1) {
-		fprintf (stderr, "Can't fork\n");
-		return 1;
-	}
-
-	if (pid == 0) {
-		err = test5 ();
-
-		sam_finalize ();
-		return (err);
-	}
-
-	waitpid (pid, &stat, 0);
-	fprintf (stderr, "test5 %s\n", (WEXITSTATUS (stat) == 0 ? "passed" : "failed"));
-	if (WEXITSTATUS (stat) != 0)
-		all_passed = 0;
-
-	pid = fork ();
-
-	if (pid == -1) {
-		fprintf (stderr, "Can't fork\n");
-		return 1;
-	}
-
-	if (pid == 0) {
-		err = test6 ();
-		sam_finalize ();
-		return (err);
-	}
-
-	waitpid (pid, &stat, 0);
-	fprintf (stderr, "test6 %s\n", (WEXITSTATUS (stat) == 0 ? "passed" : "failed"));
-	if (WEXITSTATUS (stat) != 0)
-		all_passed = 0;
-
-	pid = fork ();
-
-	if (pid == -1) {
-		fprintf (stderr, "Can't fork\n");
-		return 2;
-	}
-
-	if (pid == 0) {
-		err = test7 ();
-		sam_finalize ();
-		return (err);
-	}
-
-	waitpid (pid, &stat, 0);
-	fprintf (stderr, "test7 %s\n", (WEXITSTATUS (stat) == 0 ? "passed" : (WEXITSTATUS (stat) == 1 ? "skipped" : "failed")));
-	if (WEXITSTATUS (stat) == 1)
-		no_skipped++;
-	if (WEXITSTATUS (stat) > 1)
-		all_passed = 0;
-
-	pid = fork ();
-
-	if (pid == -1) {
-		fprintf (stderr, "Can't fork\n");
-		return 2;
-	}
-
-	if (pid == 0) {
-		err = test8 (getpid (), 0, 1);
-		sam_finalize ();
-		return (err);
-	}
-
-	waitpid (pid, &stat, 0);
-	old_pid = pid;
-
-	if (WEXITSTATUS (stat) == 0) {
+	for (n = 1; n <= no_tests; n++) {
 		pid = fork ();
 
 		if (pid == -1) {
 			fprintf (stderr, "Can't fork\n");
-			return 2;
+
+			exit (2);
 		}
 
 		if (pid == 0) {
-			err = test8 (getpid (), old_pid, 2);
+			err = test_func (getpid (), old_pid, n);
 			sam_finalize ();
-			return (err);
+			exit (err);
 		}
 
 		waitpid (pid, &stat, 0);
 		old_pid = pid;
 
-		if (WEXITSTATUS (stat) == 0) {
-			pid = fork ();
-
-			if (pid == -1) {
-				fprintf (stderr, "Can't fork\n");
-				return 2;
-			}
-
-			if (pid == 0) {
-				err = test8 (old_pid, 0, 3);
-				sam_finalize ();
-				return (err);
-			}
-
-			waitpid (pid, &stat, 0);
+		err = WEXITSTATUS (stat);
+		if (err != 0) {
+			break;
 		}
 	}
 
-	fprintf (stderr, "test8 %s\n", (WEXITSTATUS (stat) == 0 ? "passed" : (WEXITSTATUS (stat) == 1 ? "skipped" : "failed")));
-	if (WEXITSTATUS (stat) == 1)
-		no_skipped++;
-	if (WEXITSTATUS (stat) > 1)
-		all_passed = 0;
+	res_str = (err == 0 ? "passed" : (err == 1 ? "skipped" : "failed"));
 
-	pid = fork ();
-
-	if (pid == -1) {
-		fprintf (stderr, "Can't fork\n");
-		return 2;
+	if (err == 1) {
+		(*no_skipped)++;
+	} else if (err > 1) {
+		(*all_passed) = 0;
 	}
 
-	if (pid == 0) {
-		err = test9 (getpid (), 0, 1);
-		sam_finalize ();
-		return (err);
-	}
+	fprintf (stderr, "%s test %s\n", test_name, res_str);
 
-	waitpid (pid, &stat, 0);
-	old_pid = pid;
+	return (err);
+}
 
-	if (WEXITSTATUS (stat) == 0) {
-		pid = fork ();
+int main(int argc, char *argv[])
+{
+	int all_passed = 1;
+	int no_skipped = 0;
 
-		if (pid == -1) {
-			fprintf (stderr, "Can't fork\n");
-			return 2;
-		}
+	setlinebuf(stdout);
 
-		if (pid == 0) {
-			err = test9 (old_pid, 0, 2);
-			sam_finalize ();
-			return (err);
-		}
-
-		waitpid (pid, &stat, 0);
-	}
-	fprintf (stderr, "test9 %s\n", (WEXITSTATUS (stat) == 0 ? "passed" : (WEXITSTATUS (stat) == 1 ? "skipped" : "failed")));
-	if (WEXITSTATUS (stat) == 1)
-		no_skipped++;
-
-	if (WEXITSTATUS (stat) > 1)
-		all_passed = 0;
+	run_test("basics", test_basics, 0, &all_passed, &no_skipped);
+	run_test("signal", test_signal, 0, &all_passed, &no_skipped);
+	run_test("smoke", test_smoke, 0, &all_passed, &no_skipped);
+	run_test("data_store/restore", test_data_store_restore, 0, &all_passed, &no_skipped);
+	run_test("hc", test_hc, 0, &all_passed, &no_skipped);
+	run_test("warn_signal", test_warn_signal, 0, &all_passed, &no_skipped);
+	run_test("quorum", test_quorum, 1, &all_passed, &no_skipped);
+	run_test("finalize deadlock", test_finalize_deadlock, 0, &all_passed, &no_skipped);
+	run_cmap_test("cmap quit", test_cmap_quit, 3, &all_passed, &no_skipped);
+	run_cmap_test("cmap restart", test_cmap_restart, 2, &all_passed, &no_skipped);
 
 	if (all_passed)
 		fprintf (stderr, "All tests passed (%d skipped)\n", no_skipped);
