@@ -115,7 +115,7 @@ static int print_help(void)
 	printf("    about the networking and IPC traffic in some detail.\n");
 	printf("\n");
 	printf("Clear stats:\n");
-	printf("    corosync-cmapctl -C [knet|ipc|totem|all]\n");
+	printf("    corosync-cmapctl -C [knet|ipc|totem|schedmiss|all]\n");
 	printf("    The 'stats' map is implied\n");
 	printf("\n");
 	printf("Load settings from a file:\n");
@@ -316,6 +316,12 @@ static void print_key(cmap_handle_t handle,
 	if (err != CS_OK) {
 		fprintf(stderr, "Can't get value of %s. Error %s\n", key_name, cs_strerror(err));
 
+		/*
+		 * bin_value was newly allocated
+		 */
+		if (bin_value != NULL && value == NULL) {
+			free(bin_value);
+		}
 		return ;
 	}
 
@@ -411,13 +417,14 @@ static void print_key(cmap_handle_t handle,
 	printf("\n");
 }
 
-static void print_iter(cmap_handle_t handle, const char *prefix)
+static int print_iter(cmap_handle_t handle, const char *prefix)
 {
 	cmap_iter_handle_t iter_handle;
 	char key_name[CMAP_KEYNAME_MAXLEN + 1];
 	size_t value_len;
 	cmap_value_types_t type;
 	cs_error_t err;
+	int no_result = 1;
 
 	err = cmap_iter_init(handle, prefix, &iter_handle);
 	if (err != CS_OK) {
@@ -426,9 +433,12 @@ static void print_iter(cmap_handle_t handle, const char *prefix)
 	}
 
 	while ((err = cmap_iter_next(handle, iter_handle, key_name, &value_len, &type)) == CS_OK) {
+		no_result = 0;
 		print_key(handle, key_name, value_len, NULL, type);
 	}
+
 	cmap_iter_finalize(handle, iter_handle);
+	return no_result;
 }
 
 static void delete_with_prefix(cmap_handle_t handle, const char *prefix)
@@ -778,6 +788,10 @@ static void read_in_config_file(cmap_handle_t handle, char * filename)
 		} else {
 			key_type_s = strtok(NULL, " \n");
 			key_value_s = strtok(NULL, " \n");
+			if (key_type_s == NULL || key_value_s == NULL) {
+				fprintf(stderr, "Both type and value for key %s are required\n", key_name);
+				exit (EXIT_FAILURE);
+			}
 			set_key(handle, key_name, key_type_s, key_value_s);
 		}
 	}
@@ -788,9 +802,16 @@ static void read_in_config_file(cmap_handle_t handle, char * filename)
 static void clear_stats(cmap_handle_t handle, char *clear_opt)
 {
 	char key_name[CMAP_KEYNAME_MAXLEN + 1];
+	cs_error_t err;
 
 	sprintf(key_name, "stats.clear.%s", clear_opt);
-	cmap_set_uint32(handle, key_name, 1);
+
+	if ((err = cmap_set_uint32(handle, key_name, 1)) != CS_OK) {
+		fprintf(stderr, "Can't set cmap stats clear key %s. Error %s\n",
+		    key_name, cs_strerror(err));
+
+		exit (EXIT_FAILURE);
+	}
 }
 
 int main(int argc, char *argv[])
@@ -808,6 +829,7 @@ int main(int argc, char *argv[])
 	int no_retries;
 	char * clear_opt = NULL;
 	char * settings_file = NULL;
+	int count_of_no_result = 0;
 
 	action = ACTION_PRINT_PREFIX;
 	track_prefix = 1;
@@ -843,6 +865,7 @@ int main(int argc, char *argv[])
 			if (strcmp(optarg, "knet") == 0 ||
 			    strcmp(optarg, "totem") == 0 ||
 			    strcmp(optarg, "ipc") == 0 ||
+			    strcmp(optarg, "schedmiss") == 0 ||
 			    strcmp(optarg, "all") == 0) {
 				action = ACTION_CLEARSTATS;
 				clear_opt = optarg;
@@ -851,7 +874,7 @@ int main(int argc, char *argv[])
 				map = CMAP_MAP_STATS;
 			}
 			else {
-				fprintf(stderr, "argument to -C should be 'knet', 'totem', 'ipc' or 'all'\n");
+				fprintf(stderr, "argument to -C should be 'knet', 'totem', 'ipc', 'schedmiss' or 'all'\n");
 				return (EXIT_FAILURE);
 			}
 			break;
@@ -911,12 +934,16 @@ int main(int argc, char *argv[])
 	switch (action) {
 	case ACTION_PRINT_PREFIX:
 		if (argc == 0) {
-			print_iter(handle, NULL);
+			count_of_no_result = print_iter(handle, NULL);
 		} else {
 			for (i = 0; i < argc; i++) {
-				print_iter(handle, argv[i]);
+				count_of_no_result += print_iter(handle, argv[i]);
 			}
 		}
+
+		if (count_of_no_result > 0 && count_of_no_result >= argc) {
+			return (EXIT_FAILURE);
+		}
 		break;
 	case ACTION_GET:
 		for (i = 0; i < argc; i++) {
@@ -925,6 +952,7 @@ int main(int argc, char *argv[])
 				print_key(handle, argv[i], value_len, NULL, type);
 			} else {
 				fprintf(stderr, "Can't get key %s. Error %s\n", argv[i], cs_strerror(err));
+				return (EXIT_FAILURE);
 			}
 		}
 		break;
@@ -933,6 +961,7 @@ int main(int argc, char *argv[])
 			err = cmap_delete(handle, argv[i]);
 			if (err != CS_OK) {
 				fprintf(stderr, "Can't delete key %s. Error %s\n", argv[i], cs_strerror(err));
+				return (EXIT_FAILURE);
 			}
 		}
 		break;
